#include <rtl2/bc.h>
#include <rtl2/rt.h>
#include <stdio.h>

//Sleep
#ifdef WIN32
#include <Windows.h>
#endif
#ifdef __linux__
#include <winTime.h>
#endif

#define SLEEP_TIME 1

int configure_rt(msp_DEVHANDLE *);

static msp_FLAGID bcflags[] =
{
	mspF_ENHANCED_MODE,
	mspF_256WORD_BOUNDARY_DISABLE,
	mspF_MESSAGE_GAP_TIMER_ENABLED,
	mspF_INTERNAL_TRIGGER_ENABLED,
	mspF_EXPANDED_BC_CONTROL_WORD,
	0
};

void init_buffer(msp_WORD *buffer, msp_WORD val)
{
	int i;
	for (i = 0; i < 32; i++)
		buffer[i] = val;
}

int main(int argc, char *argv[])
{
	msp_DEVHANDLE bc, rt;
	msp_FRMHANDLE fr;
	msp_MSGHANDLE msg1;
	msp_Message M;
	msp_WORD val = 0;
	msp_WORD buf[32];
	int msgctr = 0;
	msp_ERROR E;
	int i;

	init_buffer(buf, val);

	msp_Startup();

	bc = msp_Open(0);
	if (!bc)
	{
		printf("Failed open bc\n");
		return 1;
	}

	msp_Configure(bc, msp_MODE_BC+msp_MODE_ENHANCED, bcflags, NULL);
	fr = msp_CreateFrame(bc, 1000, 2);
	if (!fr)
	{
		printf("Failed create frame\n");
		return 1;
	}
	msp_AddMessage(fr,
		msg1 = msp_CreateMessage(bc,
			msp_BCtoRT(&M, 1, 30, 32, buf, msp_BCCW_CHANNEL_A)),
			10000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_RTtoBC(&M, 1, 30, 32, msp_BCCW_CHANNEL_A)),
			10000);

	msp_LoadFrame(fr, msp_AUTOREPEAT);

	if ( configure_rt(&rt) != 0)
		return 1;

	msp_Start(bc);

	while (msgctr++ < 40)
	{
		E = msp_RetrieveMessage(fr, msp_NEXT_MESSAGE, &M);
		while (E == msp_ERROR_NO_NEXT_MESSAGE || E == msp_ERROR_MESSAGE_IN_PROGRESS)
		{
			Sleep(SLEEP_TIME);
			E = msp_RetrieveMessage(fr, msp_NEXT_MESSAGE, &M);
		}
		if (E != 0)
		{
			printf("Error (%d) getting a message\n", E);
			return 1;
		}

		printf("%d ", msgctr);
		switch(M.type)
		{
		case mspM_BCtoRT:
			printf("Sent ");
			break;
		case mspM_RTtoBC:
			printf("Recv ");
			break;
		default:
			printf("Unexpected ");
		}

		printf("[%d] ", M.dataWordCount);
		for (i = 0; i < M.dataWordCount; i++)
			printf("%04X ", M.Data[i]);
		printf("\n");

		if (M.type == mspM_BCtoRT)
		//Update value to send
		{
			init_buffer(buf, ++val);
			msp_WriteMessageData(msg1, buf, 32);
		}

	}

	msp_Reset(bc);
	msp_Reset(rt);

	msp_Close(bc);
	msp_Close(rt);

	msp_Cleanup();
}

static msp_FLAGID rtflags[] =
{
	mspF_ENHANCED_MODE,
	mspF_ENHANCED_RT_MEMORY_MANAGEMENT,
	//mspF_SEPARATE_BROADCAST_DATA,
	mspF_NOT_BUSY,
	mspF_NOT_SERVICE_REQUEST,
	mspF_NOT_SUBSYSTEM_FLAG,
	mspFE_NOT_SSFLAG,
	mspF_NOT_RTFLAG,
	0
};


#define LOOKUP_A	0x0140

int configure_rt(msp_DEVHANDLE *rt_)
{
	msp_DEVHANDLE rt;
	msp_BUFHANDLE bi, bo;
	msp_STKHANDLE st;

	rt = msp_Open(1);
	if (!rt)
	{
		printf("Failed open rt\n");
		return 1;
	}
	*rt_ = rt;

	msp_Configure(rt, msp_MODE_RT+msp_MODE_ENHANCED, rtflags, NULL);

	st = msp_CreateStack(rt);
	if (!st)
	{
		printf("Failed Create Stack\n");
		msp_Reset(rt);
		return 1;
	}
	msp_RTLoadStack(st, 0);

	msp_SetVReg(rt, mspRE_RT_ADDRESS, 1); // address RT

	bi = msp_CreateBuffer(rt, msp_SA_SINGLE_BUFFER, 0);
	bo = msp_CreateBuffer(rt, msp_SA_SINGLE_BUFFER, 0);

	msp_ConfigureSA(rt, 30,
		bi,
		bo,
		NULL,
		0);

	// currently RTL2 does not support circular return, so we configure it with a hack
	msp_WriteRam(rt,
		LOOKUP_A /*RT lookup pointers (area A)*/ + 32 /*TX buffers*/ + 30 /*subaddress 30*/,
		msp_AddressOf(bi)
		);

	return 0;
}
