#include <rtl2/bc.h>
#include <rtl2/rt.h>
#include <stdio.h>

//Sleep
#ifdef WIN32
#include <Windows.h>
#endif
#ifdef __linux__
#include <winTime.h>
#endif

int configure_rt(msp_DEVHANDLE *);

static msp_FLAGID bcflags[] =
{
	mspF_ENHANCED_MODE,
	mspF_256WORD_BOUNDARY_DISABLE,
	mspF_MESSAGE_GAP_TIMER_ENABLED,
	mspF_INTERNAL_TRIGGER_ENABLED,
	mspF_EXPANDED_BC_CONTROL_WORD,
	0
};

void init_buffer(msp_WORD *buffer, msp_WORD val)
{
	int i;
	for (i = 0; i < 32; i++)
		buffer[i] = val;
}

#define FR_SIZE 10
#define SLEEP_TIME 50 //ms

int main(int argc, char *argv[])
{
	msp_DEVHANDLE bc, rt;
	msp_FRMHANDLE fr;
	msp_MSGHANDLE msg;
	msp_Message M;
	msp_WORD val = 0;
	msp_WORD buf[32];
	int msgctr = 0;
	msp_ERROR E;
	int i;
	msp_WORD pTT = 0;

	msp_Startup();

	bc = msp_Open(0);
	if (!bc)
	{
		printf("Failed open bc\n");
		return 1;
	}

	msp_Configure(bc, msp_MODE_BC+msp_MODE_ENHANCED, bcflags, NULL);
	fr = msp_CreateFrame(bc, 1000, FR_SIZE);
	if (!fr)
	{
		printf("Failed create frame\n");
		return 1;
	}
	for (i = 0; i < FR_SIZE; i++)
	{
		init_buffer(buf, val++);
		msp_AddMessage(fr,
			msp_CreateMessage(bc,
				msp_BCtoRT(&M, 1, 1, 32, buf, msp_BCCW_CHANNEL_A)),
				1000);
	}

	msp_LoadFrame(fr, msp_AUTOREPEAT);

	if ( configure_rt(&rt) != 0)
		return 1;

	msp_Start(bc);

	while (msgctr < 40)
	{
		E = msp_RetrieveMessage(fr, msp_NEXT_MESSAGE, &M);

		//         .
		if (E == msp_ERROR_NO_NEXT_MESSAGE)
		{
			if (msgctr % FR_SIZE == 0)
				if (pTT != msp_ReadStackWord(fr, 0, msp_FW_TIMETAG))
					if (msp_GetMessageStatus(fr, 0) == msp_MESSAGE_COMPLETED)
						E = msp_RetrieveMessage(fr, 0, &M);
		}

		if (E == msp_ERROR_NO_NEXT_MESSAGE || E == msp_ERROR_MESSAGE_IN_PROGRESS)
		{
			Sleep(SLEEP_TIME);
			continue;
		}

		if (E != 0)
		{
			printf("Error (%d) getting a message\n", E);
			msp_Reset(bc);
			msp_Reset(rt);
			return 1;
		}

		pTT = M.timetag;

		printf("%d Sent [%d] ", msgctr, M.dataWordCount);
		for (i = 0; i < M.dataWordCount; i++)
			printf("%04X ", M.Data[i]);
		printf("\n");

		msgctr++;

		if (msgctr % FR_SIZE == 0)
		//Update value to send
		for (i = 0; i < FR_SIZE; i++)
		{
			init_buffer(buf, val++);
			msg = msp_GetFrameMessage(fr, i);
			if (!msg)
			{
				printf("Failed msp_GetFrameMessage()\n");
				msp_Reset(bc);
				msp_Reset(rt);
				return 1;
			}
			msp_WriteMessageData(msg, buf, 32);
		}
	}

	msp_Reset(bc);
	msp_Reset(rt);

	msp_Close(bc);
	msp_Close(rt);

	msp_Cleanup();

	return 0;
}

static msp_FLAGID rtflags[] =
{
	mspF_ENHANCED_MODE,
	mspF_ENHANCED_RT_MEMORY_MANAGEMENT,
	mspF_SEPARATE_BROADCAST_DATA,
	mspF_NOT_BUSY,
	mspF_NOT_SERVICE_REQUEST,
	mspF_NOT_SUBSYSTEM_FLAG,
	mspFE_NOT_SSFLAG,
	mspF_NOT_RTFLAG,
	0
};

int configure_rt(msp_DEVHANDLE *rt_)
{
	msp_DEVHANDLE rt;
	msp_STKHANDLE st;

	rt = msp_Open(1);
	if (!rt)
	{
		printf("Failed open rt\n");
		return 1;
	}
	*rt_ = rt;

	msp_Configure(rt, msp_MODE_RT+msp_MODE_ENHANCED, rtflags, NULL);

	st = msp_CreateStack(rt);
	if (!st)
	{
		printf("Failed Create Stack\n");
		msp_Reset(rt);
		return 1;
	}
	msp_RTLoadStack(st, 0);

	msp_SetVReg(rt, mspRE_RT_ADDRESS, 1); // address RT

	msp_SetSABuffer(rt, 1, msp_DIR_RX, msp_CreateBuffer(rt, msp_SA_CIRCULAR_BUFFER_2048, 0), 0); // Buffer

	return 0;
}
