/*
 * This is an multithreaded example of transmitting with DOUBLE buffer
 */

#include <rtl2/config.h>
#include <rtl2/rt.h>
#include <rtl2/bc.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#if defined (WIN32)
#include <windows.h>
#endif
#if defined(__linux__)
#include <pthread.h>
#include "../common/winTime.h"
#endif

#define RT_SND 1
#define RT_RCV 2

#define BUFFER_SIZE msp_SA_DOUBLE_BUFFER
//#define BUFFER_SIZE msp_SA_SINGLE_BUFFER

#define HW_STACK_ACCESS 0

struct thread_arg {
	int dlen;
	msp_DEVHANDLE dev;
};

#if defined(_WINDOWS)
DWORD WINAPI rt0_thread_fn(LPVOID p)
#else
void* rt0_thread_fn(void* p)
#endif
{
	struct thread_arg *t_arg = (struct thread_arg *)p;
	int ret, sent_wc = 0, dlen = t_arg->dlen;
	msp_DEVHANDLE dev_rt0 = t_arg->dev;
	msp_BUFHANDLE sabuf0;
	msp_WORD stackp = 0, stackp2 = 0;
	msp_STKHANDLE stack0 = NULL;
	msp_WORD mydata[32];
	int msgctr = 0, i;

	msp_FLAGID RTflags[] = {
	    mspF_ENHANCED_MODE,
		mspF_ENHANCED_RT_MEMORY_MANAGEMENT,
		mspF_DOUBLE_BUFFERING_ENABLE,
		mspF_SEPARATE_BROADCAST_DATA,
		mspF_NOT_BUSY,
		mspF_NOT_SERVICE_REQUEST,
		mspF_NOT_SUBSYSTEM_FLAG,
		mspFE_NOT_SSFLAG,
		mspF_NOT_RTFLAG,
		0
	};

/* configure RT_SND */
	msp_Configure(dev_rt0, msp_MODE_RT + msp_MODE_ENHANCED, RTflags, NULL);
	msp_SetVReg(dev_rt0, mspR_COMMAND_STACK_SIZE, msp_STACK_SIZE_64);
	msp_SetVReg(dev_rt0, mspRE_RT_ADDRESS, RT_SND);
	stack0 = msp_CreateStack(dev_rt0);
	sabuf0 = msp_CreateBuffer(dev_rt0, BUFFER_SIZE, 0);
	msp_FillHandle(sabuf0, 0);
	msp_SetSABuffer(dev_rt0, 1, msp_DIR_TX, sabuf0, msp_SA_WAIT_IF_BUSY);
	msp_RTLoadStack(stack0, 0);

/* write data to transmit */
	for (i = 0; i < 32; i++) mydata[i] = (msgctr << 8) + i;
	ret = msp_WriteSAData(sabuf0, mydata, 0, 32, msp_SA_INACTIVE_BUFFER | msp_SA_SWAP_BUFFER);
	sent_wc += ret;
	msgctr++;

#if HW_STACK_ACCESS
	stackp = msp_ReadRam(dev_rt0, msp_RT_STACK_POINTER_A);
#else
	stackp = msp_GetCurrentEntry(stack0, msp_LAST_RECEIVED);
#endif
	do {
#if HW_STACK_ACCESS
		while (stackp == (stackp2 = msp_ReadRam(dev_rt0, msp_RT_STACK_POINTER_A)))
			Sleep(1);

		stackp = stackp2;
		while (((ret = msp_ReadRam(dev_rt0, stackp - 4)) & msp_BSW_EOM) == 0) {
			Sleep(1);
			if (ret == 0)
				break;
		}
#else
		int ms;

		while (stackp == (stackp2 = msp_GetCurrentEntry(stack0, msp_LAST_RECEIVED))) {
			Sleep(1);
		}

		stackp = stackp2;
		while ((ms = msp_GetMessageStatus(stack0, stackp)) != msp_MESSAGE_COMPLETED) {
			Sleep(1);
		}
#endif
		for (i = 0; i < 32; i++) mydata[i] = (msgctr << 8) + i;
		ret = msp_WriteSAData(sabuf0, mydata, 0, 32, msp_SA_INACTIVE_BUFFER | msp_SA_SWAP_BUFFER);
		sent_wc += ret;
		msgctr++;
	} while (sent_wc < dlen);

#if (_WINDOWS)
	return 0;
#else
	return NULL;
#endif
}

#if defined(_WINDOWS)
DWORD WINAPI rt1_thread_fn(LPVOID p)
#else
void* rt1_thread_fn(void* p)
#endif
{
	struct thread_arg *t_arg = (struct thread_arg *)p;
	int ret, recv_wc = 0, dlen = t_arg->dlen, i, ok, success = 1;
	msp_DEVHANDLE dev_rt1 = t_arg->dev;
	msp_BUFHANDLE sabuf1;
	msp_WORD databuf[32];
	msp_WORD stackp = 0, stackp2 = 0;
	msp_STKHANDLE stack1 = NULL;

	msp_FLAGID RTflags[] = {
	    mspF_ENHANCED_MODE,
		mspF_ENHANCED_RT_MEMORY_MANAGEMENT,
		mspF_DOUBLE_BUFFERING_ENABLE,
		mspF_SEPARATE_BROADCAST_DATA,
		mspF_NOT_BUSY,
		mspF_NOT_SERVICE_REQUEST,
		mspF_NOT_SUBSYSTEM_FLAG,
		mspFE_NOT_SSFLAG,
		mspF_NOT_RTFLAG,
		0
	};

/* configure RT_RCV */
	msp_Configure(dev_rt1, msp_MODE_RT + msp_MODE_ENHANCED, RTflags, NULL);
	msp_SetVReg(dev_rt1, mspR_COMMAND_STACK_SIZE, msp_STACK_SIZE_64);
	msp_SetVReg(dev_rt1, mspRE_RT_ADDRESS, RT_RCV);
	stack1 = msp_CreateStack(dev_rt1);
	sabuf1 = msp_CreateBuffer(dev_rt1, BUFFER_SIZE, 0);
	msp_FillHandle(sabuf1, 0);
	msp_SetSABuffer(dev_rt1, 1, msp_DIR_RX, sabuf1, msp_SA_WAIT_IF_BUSY);
	msp_RTLoadStack(stack1, 0);
#if HW_STACK_ACCESS
	stackp = msp_ReadRam(dev_rt1, msp_RT_STACK_POINTER_A);
#else
	stackp = msp_GetCurrentEntry(stack1, msp_LAST_RECEIVED);
#endif
	do {
#if HW_STACK_ACCESS
		while (stackp == (stackp2 = msp_ReadRam(dev_rt1, msp_RT_STACK_POINTER_A)))
			Sleep(1);

		stackp = stackp2;
		while (((ret = msp_ReadRam(dev_rt1, stackp - 4)) & msp_BSW_EOM) == 0) {
			Sleep(1);
			if (ret == 0)
				break;
		}
#else
		while (stackp == (stackp2 = msp_GetCurrentEntry(stack1, msp_LAST_RECEIVED))) {
			Sleep(1);
		}

		stackp = stackp2;
		while (msp_GetMessageStatus(stack1, stackp) != msp_MESSAGE_COMPLETED) {
			Sleep(1);
		}
#endif

/* read data from RT2's subaddress */
		ret = msp_ReadSAData(sabuf1, databuf, 0, 32, msp_SA_INACTIVE_BUFFER | msp_SA_SAFE_READ);
		ok = 1;
		for (i = 1; i < 32; i++)
			if (databuf[i] != databuf[i-1]+1)
			{
				printf("FAIL %d %04X %04X\n", i, databuf[i-1], databuf[i]);
				ok = 0;
				success = 0;
				break;
			}
		if (!ok)
		{
			printf("NOT INNTEGRAL MESSAGE\n");
			for (i = 0; i < 32; i++) printf("%04X ",databuf[i]);
			printf("\n");
		}
		recv_wc += ret;
	} while (recv_wc < dlen);

	if (success)
		printf("SUCCESS\n");
	else
		printf("FAILURE\n");

#if (_WINDOWS)
	return 0;
#else
	return NULL;
#endif
}

int main(int argc, char *argv[])
{
	msp_DEVHANDLE dev_rt0 = NULL, dev_rt1 = NULL, dev_bc = NULL;
	int dlen, ret = 0;
	msp_FRMHANDLE F;
	msp_Message msg, *pmsg;
	msp_MSGHANDLE msghandle;
#if defined(WIN32) || defined(_WIN64)
	HANDLE    rt0_thread = NULL, rt1_thread = NULL;
#else
	pthread_t rt0_thread = 0, rt1_thread = 0;
#endif
	struct thread_arg arg0, arg1;

	msp_FLAGID BCflags[] = {
		mspF_256WORD_BOUNDARY_DISABLE,
		mspF_INTERNAL_TRIGGER_ENABLED,
		mspF_MESSAGE_GAP_TIMER_ENABLED,
		0
	};

	if (argc == 1)
		dlen = 48;
	else if (argc == 2) {
		char *endptr = NULL;
		dlen = strtol(argv[1], &endptr, 10);
		if (errno || strlen(argv[1]) != (size_t)(endptr - argv[1]) || dlen < 1) {
			printf ("error: argument is not a natural number\n");
			printf ("usage: %s [N]\n", argv[0]);
			return 1;
		}
	} else {
		printf ("usage: %s [N]\n", argv[0]);
		return 1;
	}

	arg0.dlen = arg1.dlen = dlen;

	msp_Startup();

/* open devices */
	dev_rt0 = msp_Open(0);
	if (dev_rt0 == NULL) {
		ret = 1;
		goto out;
	}
	msp_Reset(dev_rt0);
	arg0.dev = dev_rt0;

	dev_rt1 = msp_Open(1);
	if (dev_rt1 == NULL) {
		msp_Close(dev_rt0);
		ret = 2;
		goto out;;
	}
	msp_Reset(dev_rt1);
	arg1.dev = dev_rt1;

	dev_bc = msp_Open(2);
	if (dev_bc == NULL) {
		msp_Close(dev_rt0);
		msp_Close(dev_rt1);
		ret = 3;
		goto out;
	}
	msp_Reset(dev_bc);

#if defined(WIN32) || defined(_WIN64)
	rt0_thread = CreateThread(NULL, 0, rt0_thread_fn, &arg0, 0, NULL);
	if (rt0_thread == NULL) {
		ret = 1;
		printf("failed to create rt0_thread\n");
		goto exit;
	}
	rt1_thread = CreateThread(NULL, 0, rt1_thread_fn, &arg1, 0, NULL);
	if (rt1_thread == NULL) {
		ret = 1;
		printf("failed to create rt1_thread\n");
		goto exit;
	}
#else
	if (pthread_create(&rt0_thread, NULL, rt0_thread_fn, &arg0)) {
		ret = 1;
		printf("failed to create rt0_thread\n");
		goto exit;
	}
	if (pthread_create(&rt0_thread, NULL, rt1_thread_fn, &arg1)) {
		ret = 1;
		printf("failed to create rt1_thread\n");
		pthread_cancel(rt0_thread);
		goto exit;
	}
#endif

	Sleep(1000);

/* configure BC */
	msp_Configure(dev_bc, msp_MODE_BC | msp_MODE_ENHANCED, BCflags, NULL);
	msp_SetFlag(dev_bc, mspF_INTERNAL_TRIGGER_ENABLED, 1);
	msp_SetFlag(dev_bc, mspF_MESSAGE_GAP_TIMER_ENABLED, 1);
    msp_SetVReg(dev_bc, mspR_COMMAND_STACK_SIZE, msp_STACK_SIZE_64);
	F = msp_CreateFrame(dev_bc, 10, 1);
	pmsg = msp_RTtoRT(&msg, RT_SND, 1, RT_RCV, 1, 32, 0);
	msghandle = msp_CreateMessage(dev_bc, pmsg);
	msp_AddMessage(F, msghandle, 0);

	msp_LoadFrame(F, msp_AUTOREPEAT);
	msp_Start(dev_bc);

/* wait for threads to finish */
#if _WINDOWS
	WaitForSingleObject(rt0_thread, INFINITE);
	WaitForSingleObject(rt1_thread, INFINITE);
#else
	pthread_join(rt0_thread, NULL);
	pthread_join(rt1_thread, NULL);
#endif

	msp_Reset(dev_rt0);
	msp_Reset(dev_rt1);
	msp_Reset(dev_bc);

/* free resources & close devices */
	msp_DestroyFrame(F);
	msp_DestroyMessage(msghandle);

	msp_Close(dev_rt0);
	msp_Close(dev_rt1);
	msp_Close(dev_bc);

out:
	msp_Cleanup();
	if (ret && ret <= 3)
		printf("Cannot open device %d\n", ret);

exit:
	return ret;
}

