/*
 * This is an example of usage of illegal commands and busy subaddrsses
 */

#include <rtl2/config.h>
#include <rtl2/rt.h>
#include <rtl2/bc.h>
#include <stdio.h>

#if defined (WIN32)
#include <windows.h>
#endif
#if defined(__linux__)
#include "../common/winTime.h"
#endif

#define RT 1
#define BC 2

#define SA1 1
#define SA2 2
#define SA3 3
#define SA4 4

#define DATA_LEN 4

int main()
{
	msp_ERROR err = 0;
	msp_DEVHANDLE dev_rt0 = NULL, dev_bc = NULL;
	msp_WORD pattern2[DATA_LEN], pattern0[DATA_LEN], pattern1[DATA_LEN];
	int i, n = 0, ret = 0;
	msp_FRMHANDLE F;
	msp_Message *pmsg, msg, msg0, msg1, rcvd_msg;
	msp_MSGHANDLE msg0_hnd, msg1_hnd, msg2_hnd, msg3_hnd;
	msp_BUFHANDLE sabuf0, sabuf1, sabuf2;
	msp_STKHANDLE stack0 = NULL;
	msp_FLAGID RTflags[] = {
	    mspF_ENHANCED_MODE,
		mspF_ENHANCED_RT_MEMORY_MANAGEMENT,
		mspF_BUSY_BIT_LOOKUP_TABLE_ENABLE,
		mspF_SEPARATE_BROADCAST_DATA,
		mspF_NOT_BUSY,
		mspF_NOT_SERVICE_REQUEST,
		mspF_NOT_SUBSYSTEM_FLAG,
		mspF_NOT_RTFLAG,
		mspFE_NOT_SSFLAG,
		0
	};
	msp_FLAGID BCflags[] = { 
		mspF_256WORD_BOUNDARY_DISABLE,
		mspF_INTERNAL_TRIGGER_ENABLED,
		mspF_MESSAGE_GAP_TIMER_ENABLED,
		0
	};

	for (i = 0; i < DATA_LEN; i++) {
		pattern0[i] = 0x1000 + i;
		pattern1[i] = 0x2000 + i;
		pattern2[i] = 0x3000 + i;
	}

	msp_Startup();

/* open devices */
	dev_rt0 = msp_Open(0);
	if (dev_rt0 == NULL) {
		ret = 1;
		goto out;
	}

	dev_bc = msp_Open(1);
	if (dev_bc == NULL) {
		msp_Close(dev_rt0);
		ret = 3;
		goto out;
	}

	msp_Reset(dev_rt0);
	msp_Reset(dev_bc);


/* configure RT */
	msp_Configure(dev_rt0, msp_MODE_RT + msp_MODE_ENHANCED, RTflags, NULL);
	msp_SetVReg(dev_rt0, mspR_COMMAND_STACK_SIZE, msp_STACK_SIZE_64);
	msp_SetVReg(dev_rt0, mspRE_RT_ADDRESS, RT);
	err = msp_SetFlag(dev_rt0, mspFS_RT_SET_BUSY_FOR_BUFFER, 1);
	err = msp_SetFlag(dev_rt0, mspFS_RT_ILLEGALIZE_FOR_BUFFER, 1);

	stack0 = msp_CreateStack(dev_rt0);

	sabuf0 = msp_CreateBuffer(dev_rt0, msp_SA_SINGLE_BUFFER, 0);
	msp_FillHandle(sabuf0, 0);
	msp_SetSABuffer(dev_rt0, SA1, msp_DIR_TX, sabuf0, 0);

	sabuf1 = msp_CreateBuffer(dev_rt0, msp_SA_SINGLE_BUFFER, 0);
	msp_FillHandle(sabuf1, 0);
	msp_SetSABuffer(dev_rt0, SA2, msp_DIR_TX, sabuf1, 0);

	sabuf2 = msp_CreateBuffer(dev_rt0, msp_SA_SINGLE_BUFFER, 0);
	msp_FillHandle(sabuf2, 0);
	msp_SetSABuffer(dev_rt0, SA3, msp_DIR_TX, sabuf2, 0);

	msp_RTLoadStack(stack0, 0);

/* write data to transmit */
	msp_WriteSAData(sabuf0, pattern0, 0, DATA_LEN, 0);
	msp_WriteSAData(sabuf1, pattern1, 0, DATA_LEN, 0);
	msp_WriteSAData(sabuf2, pattern2, 0, DATA_LEN, 0);

/* Illegalise/legalise commands */
	msp_SetIllegalCommands(dev_rt0, 0x0000, 0xFFFF, msp_ILLEGAL);
	msp_IllegalizeForBuffer(sabuf0, 1, DATA_LEN);
	msp_IllegalizeForBuffer(sabuf1, 1, DATA_LEN + 2);
	msp_IllegalizeForBuffer(sabuf2, DATA_LEN, DATA_LEN);

/* Set Busy/Not Busy state */
	msp_SetBufferFlag(sabuf1, msp_SA_BUSY, msp_BUSY);
	msp_SetBufferFlag(sabuf2, msp_SA_BUSY, msp_BUSY);

/* configure BC */
	msp_Configure(dev_bc, msp_MODE_BC | msp_MODE_ENHANCED, BCflags, NULL);
    msp_SetVReg(dev_bc, mspR_COMMAND_STACK_SIZE, msp_STACK_SIZE_64);
	
	F = msp_CreateFrame(dev_bc, 1000, 4);

	pmsg = msp_RTtoBC(&msg0, RT, SA1, DATA_LEN, 0); 
	msg0_hnd = msp_CreateMessage(dev_bc, pmsg);
	msp_AddMessage(F, msg0_hnd, 1000);

	pmsg = msp_RTtoBC(&msg1, RT, SA2, DATA_LEN, 0); 
	msg1_hnd = msp_CreateMessage(dev_bc, pmsg);
	msp_AddMessage(F, msg1_hnd, 1000);

	pmsg = msp_RTtoBC(&msg, RT, SA3, DATA_LEN - 2, 0);   // alter message length to make it illegal
	msg2_hnd = msp_CreateMessage(dev_bc, pmsg);
	err = msp_AddMessage(F, msg2_hnd, 1000);

	pmsg = msp_RTtoBC(&msg1, RT, SA4, DATA_LEN, 0);		// send to unitialized subaddress to make message illegal
	msg3_hnd = msp_CreateMessage(dev_bc, pmsg);
	msp_AddMessage(F, msg3_hnd, 1000);

	err = msp_LoadFrame(F, 0);

/* running the frame */
	err = msp_Start(dev_bc);

	Sleep(1000);

	msp_Reset(dev_rt0);
	msp_Reset(dev_bc);

/* retrieve messages  */
	n = 1;
	do {
		unsigned short tx, rt, sa, wcnt, busy, illegal, good_data;
		msp_RetrieveMessage(F, msp_NEXT_MESSAGE, &rcvd_msg);

		busy = rcvd_msg.StatusWord1 & msp_STATUS_BUSY;
		illegal = rcvd_msg.StatusWord1 & msp_STATUS_MESSAGE_ERROR;
		good_data = rcvd_msg.bsw & msp_BSW_BC_GOOD_DATA_BLOCK_TRANSFER;
		ParseCommandWord(rcvd_msg.CmdWord1, tx, rt, sa, wcnt);
		printf("msg %d SA %d TX (%s%s%s) [ %d / %s ]%s", n, sa,
				busy ? "BUSY" : "",
				illegal && busy ? "+" : "",
				illegal ? "ILLEGAL" : "",
				rcvd_msg.dataWordCount,
				good_data? "GOOD_DATA" : "!GOOD_DATA",
				good_data? " : " : "");
		//printf("\nBSW %04X STATUS %04X WC %d\n", rcvd_msg.bsw, rcvd_msg.StatusWord1, rcvd_msg.dataWordCount);
		if (good_data)
			for (i = 0; i < rcvd_msg.dataWordCount; i++)
				printf("%04X ", rcvd_msg.Data[i]);
		printf("\n");
	} while (n++ < 4);

	msp_Close(dev_rt0);
	msp_Close(dev_bc);

out:
	msp_Cleanup();
	if (ret > 0 && ret < 3)
		printf("Cannot open device %d\n", ret);

	return ret;
}
