/*
 * This is an example of mode code messages with data transfer
 */
#include <rtl2/config.h>
#include <rtl2/rt.h>
#include <rtl2/bc.h>
#include <stdio.h>

#if defined (WIN32)
#include <windows.h>
#endif
#if defined(__linux__)
#include "../common/winTime.h"
#endif

#define RT 1

void configure_bc();

int main()
{
	msp_DEVHANDLE dev_rt0 = NULL, dev_bc = NULL;
	int n, N;
	msp_FRMHANDLE F;
	msp_Message msg0, rcvd_msg;
	msp_STKHANDLE stack0 = NULL;
	msp_BYTE rt, d, sa, mc;
	msp_FLAGID RTflags[] = {
	    mspF_ENHANCED_MODE,
		mspF_ENHANCED_RT_MEMORY_MANAGEMENT,
		mspF_ENHANCED_MODE_CODE_HANDLING,
		mspF_SEPARATE_BROADCAST_DATA,
		mspF_NOT_BUSY,
		mspF_NOT_SERVICE_REQUEST,
		mspF_NOT_SUBSYSTEM_FLAG,
		mspF_NOT_RTFLAG,
		mspFE_NOT_SSFLAG,
		0
	};
	msp_FLAGID BCflags[] = { 
		mspF_256WORD_BOUNDARY_DISABLE,
		mspF_INTERNAL_TRIGGER_ENABLED,
		mspF_MESSAGE_GAP_TIMER_ENABLED,
		0
	};

	msp_Startup();

/* open devices */
	dev_rt0 = msp_Open(0);
	if (dev_rt0 == NULL) {
		printf("Cannot open device 1\n");
		return 1;
	}

	dev_bc = msp_Open(1);
	if (dev_bc == NULL) {
		msp_Close(dev_rt0);
		printf("Cannot open device 2\n");
		return 2;
	}

	msp_Reset(dev_rt0);
	msp_Reset(dev_bc);

/* configure RT */
	msp_Configure(dev_rt0, msp_MODE_RT + msp_MODE_ENHANCED, RTflags, NULL);
	msp_SetVReg(dev_rt0, mspR_COMMAND_STACK_SIZE, msp_STACK_SIZE_64);
	msp_SetVReg(dev_rt0, mspRE_RT_ADDRESS, RT);

	stack0 = msp_CreateStack(dev_rt0);
	msp_RTLoadStack(stack0, 0);

/* Illegalise/legalise commands */
	msp_SetIllegalCommands(dev_rt0, 0x0000, 0xFFFF, msp_ILLEGAL);
	/* Enable all mode code commands */
	msp_SetIllegalCommands(dev_rt0, msp_rangeCmdILL(msp_RANGE_ALL, msp_RANGE_ALL, 0, 0, 0, 0x1F),  msp_LEGAL);
	msp_SetIllegalCommands(dev_rt0, msp_rangeCmdILL(msp_RANGE_ALL, msp_RANGE_ALL, 0x1F, 0x1F, 0, 0x1F),  msp_LEGAL);

/* Set data to transmit */
	msp_SetModeCodeData(dev_rt0, FormatCommandWord(msp_DIR_TX, RT, 0, mspMC_TRANSMIT_VECTOR_WORD), 0x0001);
	msp_SetModeCodeData(dev_rt0, FormatCommandWord(msp_DIR_TX, RT, 0, mspMC_TRANSMIT_BIT_WORD), 0x0002);

/* configure BC */
	msp_Configure(dev_bc, msp_MODE_BC | msp_MODE_ENHANCED, BCflags, NULL);
    msp_SetVReg(dev_bc, mspR_COMMAND_STACK_SIZE, msp_STACK_SIZE_64);
	
	F = msp_CreateFrame(dev_bc, 10, 10);

	msp_AddMessage(F, 
		msp_CreateMessage(dev_bc,
			msp_Modecode_data_tx(&msg0, RT, mspMC_TRANSMIT_VECTOR_WORD, msp_BCCW_CHANNEL_A)
		),
		100
	);

	msp_AddMessage(F, 
		msp_CreateMessage(dev_bc,
			msp_Modecode_data_tx(&msg0, RT, mspMC_TRANSMIT_BIT_WORD, msp_BCCW_CHANNEL_A)
		),
		100
	);

	msp_AddMessage(F, 
		msp_CreateMessage(dev_bc,
			msp_Modecode_data_rx(&msg0, RT, mspMC_SYNCHRONIZE_WITH_DATA, 0x8000, msp_BCCW_CHANNEL_A)
		),
		100
	);
	
	msp_AddMessage(F, 
		msp_CreateMessage(dev_bc,
			msp_Modecode_data_rx(&msg0, RT, mspMC_SELECTED_TRANSMITTER_SHUTDOWN, 0x0003, msp_BCCW_CHANNEL_A)
		),
		100
	);
	
	msp_AddMessage(F, 
		msp_CreateMessage(dev_bc,
			msp_Modecode_data_rx(&msg0, RT, mspMC_OVERRIDE_SELECTED_TRANSMITTER_SHUTDOWN, 0x0004, msp_BCCW_CHANNEL_A)
		),
		100
	);

	N = msp_GetFrameProp(F, msp_FD_MessageCount);
	msp_LoadFrame(F, 0);

/* running the frame */
	msp_Start(dev_bc);

	Sleep(1);

/* extracting data received by RT */
	printf("RT %d DATA:\n", RT);
	printf("SYNCHRONIZE_WITH_DATA received data = %04X\n",
		msp_GetModeCodeData(dev_rt0, FormatCommandWord(msp_DIR_RX, RT, 0, mspMC_SYNCHRONIZE_WITH_DATA)));
	printf("SELECTED_TRANSMITTER_SHUTDOWN received data = %04X\n",
		msp_GetModeCodeData(dev_rt0, FormatCommandWord(msp_DIR_RX, RT, 0, mspMC_SELECTED_TRANSMITTER_SHUTDOWN)));
	printf("OVERRIDE_SELECTED_TRANSMITTER_SHUTDOWN received data = %04X\n",
		msp_GetModeCodeData(dev_rt0, FormatCommandWord(msp_DIR_RX, RT, 0, mspMC_OVERRIDE_SELECTED_TRANSMITTER_SHUTDOWN)));

/* extracting data received by BC */
	printf("BC DATA:\n");
	for (n = 0; n < N; n++)
	{
		msp_RetrieveMessage(F, n, &rcvd_msg);
		ParseCommandWord(rcvd_msg.CmdWord1, d, rt, sa, mc);
		printf("MSG %d", n);
		//printf(" BSW = %04X status = %04X", rcvd_msg.bsw, rcvd_msg.StatusWord1);
		printf(" mode code = %02X", mc);
		switch(mc)
		{
		case mspMC_TRANSMIT_VECTOR_WORD:
		case mspMC_TRANSMIT_BIT_WORD:
			printf(" received data word = %04x", rcvd_msg.Data[0]);
		default:
			;
		}
		printf("\n");

	}
	printf("\n");

/* finish works */
	msp_Reset(dev_rt0);
	msp_Reset(dev_bc);

	msp_Close(dev_rt0);
	msp_Close(dev_bc);

	msp_Cleanup();

	return 0;
}

