/*
 * This is an multithreaded example of using msp_RetrieveMessage() and msp_RetrieveMessageInfo()
 */
#include <rtl2/config.h>
#include <rtl2/rt.h>
#include <rtl2/bc.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#if defined (WIN32)
#include <windows.h>
#endif
#if defined(__linux__)
#include <pthread.h>
#include "../common/winTime.h"
#endif

#define RT_SND 1
#define RT_RCV 2

#define SALT 9999

struct thread_arg {
	int dlen;
	msp_DEVHANDLE dev;
	msp_WORD *pattern;
	msp_WORD *databuf;
};

#if defined(_WINDOWS)
DWORD WINAPI rt0_thread_fn(LPVOID p)
#else
void* rt0_thread_fn(void* p)
#endif
{
	struct thread_arg *t_arg = (struct thread_arg *)p;
	int ret, sent_wc = 0, b2s, dlen = t_arg->dlen;
	msp_DEVHANDLE dev_rt0 = t_arg->dev;
	msp_BUFHANDLE sabuf0;
	msp_WORD *pattern = t_arg->pattern;
	msp_STKHANDLE stack0 = NULL;
	msp_ERROR err;
	msp_MessageInfo MI;

	msp_FLAGID RTflags[] = {
	    mspF_ENHANCED_MODE,
		mspF_ENHANCED_RT_MEMORY_MANAGEMENT,
		mspF_SEPARATE_BROADCAST_DATA,
		mspF_NOT_BUSY,
		mspF_NOT_SERVICE_REQUEST,
		mspF_NOT_SUBSYSTEM_FLAG,
		mspFE_NOT_SSFLAG,
		mspF_NOT_RTFLAG,
		0
	};

/* configure RT_SND */
	msp_Configure(dev_rt0, msp_MODE_RT + msp_MODE_ENHANCED, RTflags, NULL);
	msp_SetVReg(dev_rt0, mspR_COMMAND_STACK_SIZE, msp_STACK_SIZE_64);
	msp_SetVReg(dev_rt0, mspRE_RT_ADDRESS, RT_SND);
	stack0 = msp_CreateStack(dev_rt0);
	sabuf0 = msp_CreateBuffer(dev_rt0, msp_SA_SINGLE_BUFFER, 0);
	msp_FillHandle(sabuf0, 0);
	msp_SetSABuffer(dev_rt0, 1, msp_DIR_TX, sabuf0, msp_SA_WAIT_IF_BUSY);
	msp_RTLoadStack(stack0, 0);

/* write data to transmit */
	b2s = sent_wc + 32 > dlen ? dlen - sent_wc : 32;
	ret = msp_WriteSAData(sabuf0, pattern + sent_wc, 0, b2s, 0);
	sent_wc += ret;

	do {
		err = msp_RetrieveMessageInfo(stack0, msp_NEXT_MESSAGE, &MI);
		while (err == msp_ERROR_NO_NEXT_MESSAGE || err == msp_ERROR_MESSAGE_IN_PROGRESS)
		{
			Sleep(1);
			err = msp_RetrieveMessageInfo(stack0, msp_NEXT_MESSAGE, &MI);
		}
		if (err != msp_NOERROR)
		{
			printf("RT %d: Error (%d) getting a message\n", RT_SND, err);
#if (_WINDOWS)
			return 1;
#else
			return (void *) 1;
#endif
		}
		b2s = sent_wc + 32 > dlen ? dlen - sent_wc : 32;
		ret = msp_WriteSAData(sabuf0, pattern + sent_wc, 0, b2s, 0);
		sent_wc += ret;
	} while (sent_wc < dlen);

#if (_WINDOWS)
	return 0;
#else
	return NULL;
#endif
}

#if defined(_WINDOWS)
DWORD WINAPI rt1_thread_fn(LPVOID p)
#else
void* rt1_thread_fn(void* p)
#endif
{
	struct thread_arg *t_arg = (struct thread_arg *)p;
	int b2r, recv_wc = 0, dlen = t_arg->dlen, mwi;
	msp_DEVHANDLE dev_rt1 = t_arg->dev;
	msp_BUFHANDLE sabuf1;
	msp_WORD *databuf;
	msp_STKHANDLE stack1 = NULL;
	msp_ERROR err;
	msp_Message M;

	msp_FLAGID RTflags[] = {
	    mspF_ENHANCED_MODE,
		mspF_ENHANCED_RT_MEMORY_MANAGEMENT,
		mspF_SEPARATE_BROADCAST_DATA,
		mspF_NOT_BUSY,
		mspF_NOT_SERVICE_REQUEST,
		mspF_NOT_SUBSYSTEM_FLAG,
		mspFE_NOT_SSFLAG,
		mspF_NOT_RTFLAG,
		0
	};

	databuf = t_arg->databuf;

/* configure RT_RCV */
	msp_Configure(dev_rt1, msp_MODE_RT + msp_MODE_ENHANCED, RTflags, NULL);
	msp_SetVReg(dev_rt1, mspR_COMMAND_STACK_SIZE, msp_STACK_SIZE_64);
	msp_SetVReg(dev_rt1, mspRE_RT_ADDRESS, RT_RCV);
	stack1 = msp_CreateStack(dev_rt1);
	sabuf1 = msp_CreateBuffer(dev_rt1, msp_SA_SINGLE_BUFFER, 0);
	msp_FillHandle(sabuf1, 0);
	msp_SetSABuffer(dev_rt1, 1, msp_DIR_RX, sabuf1, msp_SA_WAIT_IF_BUSY);
	msp_RTLoadStack(stack1, 0);

	do {
		err = msp_RetrieveMessage(stack1, msp_NEXT_MESSAGE, &M);
		while (err == msp_ERROR_NO_NEXT_MESSAGE || err == msp_ERROR_MESSAGE_IN_PROGRESS)
		{
			Sleep(1);
			err = msp_RetrieveMessage(stack1, msp_NEXT_MESSAGE, &M);
		}
		if (err != msp_NOERROR)
		{
			printf("RT %d: Error (%d) getting a message\n", RT_RCV, err);
#if (_WINDOWS)
			return 1;
#else
			return (void *) 1;
#endif
		}
/* read data from received message */
		b2r = recv_wc + M.dataWordCount > dlen ? dlen - recv_wc : M.dataWordCount;
		for (mwi = 0; mwi < b2r; mwi++)
			*(databuf + recv_wc + mwi) = M.Data[mwi];
		recv_wc += b2r;
	} while (recv_wc < dlen);

#if (_WINDOWS)
	return 0;
#else
	return NULL;
#endif
}

int main(int argc, char *argv[])
{
	msp_DEVHANDLE dev_rt0 = NULL, dev_rt1 = NULL, dev_bc = NULL;
	msp_WORD *databuf, *pattern;
	int dlen, i = 0, eq = 0, ret = 0;
	msp_FRMHANDLE F;
	msp_Message msg, *pmsg;
	msp_MSGHANDLE msghandle;
#if defined(WIN32) || defined(_WIN64)
	HANDLE    rt0_thread = NULL, rt1_thread = NULL;
#else
	pthread_t rt0_thread = 0, rt1_thread = 0;
#endif
	struct thread_arg arg0, arg1;

	msp_FLAGID BCflags[] = {
		mspF_256WORD_BOUNDARY_DISABLE,
		mspF_INTERNAL_TRIGGER_ENABLED,
		mspF_MESSAGE_GAP_TIMER_ENABLED,
		0
	};

	if (argc == 1)
		dlen = 32;//48;
	else if (argc == 2) {
		char *endptr = NULL;
		dlen = strtol(argv[1], &endptr, 10);
		if (errno || strlen(argv[1]) != (size_t)(endptr - argv[1]) || dlen < 1) {
			printf ("error: argument is not a natural number\n");
			printf ("usage: %s [N]\n", argv[0]);
			return 1;
		}
	} else {
		printf ("usage: %s [N]\n", argv[0]);
		return 1;
	}

	arg0.dlen = arg1.dlen = dlen;

	pattern = (msp_WORD *) malloc(sizeof(msp_WORD) * dlen);
	databuf = (msp_WORD *) malloc(sizeof(msp_WORD) * dlen);
	for (i = 0; i < dlen; i++) {
		pattern[i] = SALT - i;
		databuf[i] = 0;
	}
	arg0.pattern = arg1.pattern = pattern;
	arg1.databuf = databuf;

	printf("pattern: ");
	for (i = 0; i < dlen; i++)
		printf("%04X ", pattern[i]);
	printf("\n\n");

	msp_Startup();

/* open devices */
	dev_rt0 = msp_Open(0);
	if (dev_rt0 == NULL) {
		ret = 1;
		goto out;
	}
	msp_Reset(dev_rt0);
	arg0.dev = dev_rt0;

	dev_rt1 = msp_Open(1);
	if (dev_rt1 == NULL) {
		msp_Close(dev_rt0);
		ret = 2;
		goto out;
	}
	msp_Reset(dev_rt1);
	arg1.dev = dev_rt1;

	dev_bc = msp_Open(2);
	if (dev_bc == NULL) {
		msp_Close(dev_rt0);
		msp_Close(dev_rt1);
		ret = 3;
		goto out;
	}
	msp_Reset(dev_bc);

#if defined(WIN32) || defined(_WIN64)
	rt0_thread = CreateThread(NULL, 0, rt0_thread_fn, &arg0, 0, NULL);
	if (rt0_thread == NULL) {
		ret = 1;
		printf("failed to create rt0_thread\n");
		goto exit;
	}
	rt1_thread = CreateThread(NULL, 0, rt1_thread_fn, &arg1, 0, NULL);
	if (rt1_thread == NULL) {
		ret = 1;
		printf("failed to create rt1_thread\n");
		goto exit;
	}
#else
	if (pthread_create(&rt0_thread, NULL, rt0_thread_fn, &arg0)) {
		ret = 1;
		printf("failed to create rt0_thread\n");
		goto exit;
	}
	if (pthread_create(&rt0_thread, NULL, rt1_thread_fn, &arg1)) {
		ret = 1;
		printf("failed to create rt1_thread\n");
		pthread_cancel(rt0_thread);
		goto exit;
	}
#endif
	Sleep(1000);

/* configure BC */
	msp_Configure(dev_bc, msp_MODE_BC | msp_MODE_ENHANCED, BCflags, NULL);
    msp_SetVReg(dev_bc, mspR_COMMAND_STACK_SIZE, msp_STACK_SIZE_64);
	F = msp_CreateFrame(dev_bc, 200, 1);
	pmsg = msp_RTtoRT(&msg, RT_SND, 1, RT_RCV, 1, 32, 0);
	msghandle = msp_CreateMessage(dev_bc, pmsg);
	msp_AddMessage(F, msghandle, 1000);

	msp_LoadFrame(F, msp_AUTOREPEAT);
	msp_Start(dev_bc);

/* wait for threads to finish */
#if _WINDOWS
	WaitForSingleObject(rt0_thread, INFINITE);
	WaitForSingleObject(rt1_thread, INFINITE);
#else
	pthread_join(rt0_thread, NULL);
	pthread_join(rt1_thread, NULL);
#endif

	msp_Reset(dev_bc);
	msp_Reset(dev_rt0);
	msp_Reset(dev_rt1);

	printf("databuf: ");
	for (i = 0; i < dlen; i++) {
		printf("%04X ", databuf[i]);
		if ((databuf[i] != pattern[i]) && (eq == 0)) {
			printf("\n%d) %04X != %04X\n", i, pattern[i], databuf[i]);
			eq = 1;
		}
	}
	printf("\n\n%s\n", eq == 1 ? "FAIL" : "OK");

	msp_Close(dev_rt0);
	msp_Close(dev_rt1);
	msp_Close(dev_bc);

out:
	msp_Cleanup();
	if (ret && ret <= 3)
		printf("Cannot open device %d\n", ret);

exit:
	free(pattern);
	free(databuf);
	return ret;
}

