#include <rtl2/mt.h>
#include <rtl2/bc.h>
#include <stdio.h>

//Sleep
#ifdef WIN32
#include <Windows.h>
#endif
#ifdef __linux__
#include <winTime.h>
#endif

#define SLEEP_TIME 50 //ms

int configure_bc(msp_DEVHANDLE);

static msp_FLAGID mtflags[] =
{
	mspF_256WORD_BOUNDARY_DISABLE,
	0
};

static msp_REGVALUE mtregs[] =
{
	{ mspR_MT_COMMAND_STACK_SIZE, msp_MT_CSTACK_SIZE_64 },
	{ mspR_MT_DATA_STACK_SIZE, msp_MT_DSTACK_SIZE_2K },
	{ 0, 0 }
};

char *msgtype2str(msp_WORD msgtype);

int main(int argc, char *argv[])
{
	msp_DEVHANDLE bc, mt;
	msp_STKHANDLE st;
	msp_DSTKHANDLE dst;
	msp_Message M;
	int msgctr = 0;
	msp_ERROR E;
	int i;

	msp_Startup();

	mt = msp_Open(0);
	if (!mt)
	{
		printf("Failed open mt\n");
		return 1;
	}

	bc = msp_Open(1);
	if (!bc)
	{
		printf("Failed open bc\n");
		return 1;
	}

	msp_Configure(mt, msp_MODE_MM+msp_MODE_ENHANCED, mtflags, mtregs);

	st = msp_MTCreateStack(mt);
	if (!st)
	{
		printf("Failed create command stack\n");
		return 1;
	}
	msp_MTLoadStack(st, 0);

	dst = msp_MTCreateDataStack(mt);
	if (!st)
	{
		printf("Failed create data stack\n");
		return 1;
	}
	msp_MTLoadDataStack(dst, 0);

#define SELECTED_MESSAGES 1
#if SELECTED_MESSAGES
	//  /  2  4  1
	msp_MTSelectMessages(mt, msp_rangeCmdMT(msp_RANGE_ALL, 1, 1, 2, 2), msp_MT_SELECTED);
	msp_MTSelectMessages(mt, msp_rangeCmdMT(msp_RANGE_ALL, 1, 1, 4, 4), msp_MT_SELECTED);

	//   /  3  4
	msp_MTSelectMessages(mt, msp_rangeCmdMT(msp_RANGE_ALL, 3, 3, 0, 31), msp_MT_SELECTED);
	msp_MTSelectMessages(mt, msp_rangeCmdMT(msp_RANGE_ALL, 4, 4, 0, 31), msp_MT_SELECTED);
#else
	msp_MTSelectMessages(mt, msp_rangeAll, msp_MT_SELECTED);
#endif

	if ( configure_bc(bc) != 0)
		return 1;

	msp_Start(mt);
	msp_Start(bc);

	while (msgctr++ < 9)
	{
		E = msp_RetrieveMessage(st, msp_NEXT_MESSAGE, &M);
		while (E  == msp_ERROR_NO_NEXT_MESSAGE || E == msp_ERROR_MESSAGE_IN_PROGRESS)
		{
			Sleep(SLEEP_TIME);
			E = msp_RetrieveMessage(st, msp_NEXT_MESSAGE, &M);
		}

		if (E != 0)
		{
			printf("Error (%d) getting a message\n", E);
			return 1;
		}

		printf("%d %s rt=%d sa=%d ", msgctr, msgtype2str(M.type), M.CmdWord1 >> 11, (M.CmdWord1 >> 5)&0x1F);
		printf("[%d] ", M.dataWordCount);
		for (i = 0; i < M.dataWordCount; i++)
			printf("%04X ", M.Data[i]);
		printf("\n");
	}

	msp_Reset(bc);
	msp_Reset(mt);

	msp_Close(bc);
	msp_Close(mt);

	msp_Cleanup();

	return 0;
}

char *msgtype2str(msp_WORD msgtype)
{
	switch(msgtype)
	{
	case mspM_BCtoRT:		return "BCtoRT";
	case mspM_RTtoBC:		return "RTtoBC";
	case mspM_RTtoRT:		return "RTtoRT";
	case mspM_BCtoRT_BROADCAST:		return "BCtoRT_BROADCAST";
	case mspM_RTtoRT_BROADCAST:		return "RTtoRT_BROADCAST";
	case mspM_MODECODE:		return "MODECODE";
	case mspM_MODECODE_DATA_TX:		return "MODECODE_DATA_TX";
	case mspM_MODECODE_DATA_RX:		return "MODECODE_DATA_RX";
	case mspM_MODECODE_BROADCAST:		return "MODECODE_BROADCAST";
	case mspM_MODECODE_DATA_BROADCAST:		return "MODECODE_DATA_BROADCAST";
	default:	return "Unknown ";
	}
}

static msp_FLAGID bcflags[] =
{
	mspF_ENHANCED_MODE,
	mspF_256WORD_BOUNDARY_DISABLE,
	mspF_MESSAGE_GAP_TIMER_ENABLED,
	mspF_INTERNAL_TRIGGER_ENABLED,
	mspF_EXPANDED_BC_CONTROL_WORD,
	0
};

void init_buffer(msp_WORD *buffer, msp_WORD val)
{
	int i;
	for (i = 0; i < 32; i++)
		buffer[i] = val;
}

int configure_bc(msp_DEVHANDLE bc)
{
	msp_FRMHANDLE fr;
	msp_Message M;
	msp_WORD val = 0;
	msp_WORD buf[32];

	init_buffer(buf, val);

	msp_Configure(bc, msp_MODE_BC+msp_MODE_ENHANCED, bcflags, NULL);
	fr = msp_CreateFrame(bc, 1000, 10);
	if (!fr)
	{
		printf("Failed create frame\n");
		return 1;
	}
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 1, 1, 1, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 1, 2, 1, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 1, 3, 1, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 1, 4, 1, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 2, 1, 1, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 3, 1, 1, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 4, 1, 1, buf, msp_BCCW_CHANNEL_A)),
			1000);

	msp_LoadFrame(fr, msp_AUTOREPEAT);
	return 0;
}
