#include <rtl2/mt.h>
#include <rtl2/bc.h>
#include <stdio.h>

//Sleep
#ifdef WIN32
#include <Windows.h>
#endif
#ifdef __linux__
#include <winTime.h>
#endif

#define SLEEP_TIME 1 //ms
#define MAX_WORDS 200

int configure_bc(msp_DEVHANDLE);

int main(int argc, char *argv[])
{
	msp_DEVHANDLE bc, mt;
	int wctr = 0;
	msp_DWORD dssize;
	msp_DWORD halfsize;
	msp_WORD rdaddr, wraddr;
	msp_WORD IDW, DW;

	msp_Startup();

	mt = msp_Open(0);
	if (!mt)
	{
		printf("Failed open mt\n");
		return 1;
	}

	bc = msp_Open(1);
	if (!bc)
	{
		printf("Failed open bc\n");
		return 1;
	}

	msp_SelectMode(mt, msp_MODE_WM+msp_MODE_ENHANCED);
	if (msp_RamTotal(mt) >= 65536)
	{
		dssize = 65536;
		msp_SetVReg(mt, mspR_MT_DATA_STACK_SIZE, msp_MT_DSTACK_SIZE_64K);
	}
	else
	{
		dssize = 4096;
		msp_SetVReg(mt, mspR_MT_DATA_STACK_SIZE, msp_MT_DSTACK_SIZE_4K);
		// For 26.20 with cache set the size to msp_MT_DSTACK_SIZE_64K anyway!
	}

	halfsize = (msp_WORD)(msp_RamTotal(mt)/2);
	msp_FillRamBLK(mt, 0, 0, halfsize);
	msp_FillRamBLK(mt, halfsize, 0, halfsize);
	rdaddr = 0;

	if ( configure_bc(bc) != 0)
		return 1;

	msp_Start(mt);
	msp_Start(bc);

	while (wctr < MAX_WORDS)
	{
		wraddr = msp_ReadReg(mt, mspRR_DATA_STACK_ADDRESS);
		if (wraddr == rdaddr || wraddr == rdaddr + 1)
		{
			Sleep(SLEEP_TIME);
			continue;
		}

		DW = msp_ReadRam(mt, rdaddr);
		IDW = msp_ReadRam(mt, rdaddr+1);

		printf("%05d ", wctr);
		printf("%04X ", DW);
		if (IDW & msp_WM_N_GAP) printf("NGAP "); else printf("%4d ", msp_WM_GAP_TIME(IDW));
		if (IDW & msp_WM_CHANNEL) printf("ch B "); else printf("ch A ");
		if (IDW & msp_WM_SYNC_TYPE) printf("CMD/ST "); else printf("data   ");
		if (IDW & msp_WM_ERROR) printf("!ERROR!");
		printf("\n");
		if (IDW & msp_WM_SYNC_TYPE)
		{
			msp_WORD tx, rt, sa, wcnt;
			ParseCommandWord(DW, tx, rt, sa, wcnt);
			printf("\t%d-%d-%d-%d\n", rt, tx, sa, wcnt);
			printf("\t%d-%03X ", StatusRTADD(DW), DW & 0x7FFF);
			if (DW & msp_STATUS_MESSAGE_ERROR) printf("ME ");
			if (DW & msp_STATUS_SERVICE_REQUEST) printf("SR ");
			if (DW & msp_STATUS_BROADCAST_COMMAND_RECEIVED) printf("BC ");
			if (DW & msp_STATUS_BUSY) printf("BY ");
			if (DW & msp_STATUS_SUBSYSTEM_FLAG) printf("SF ");
			if (DW & msp_STATUS_TERMINAL_FLAG) printf("TF ");
			if (DW & msp_STATUS_DYNAMIC_BUS_CONTROL_ACCEPTED) printf("DBCA");
			printf("\n");
		}

		rdaddr = (rdaddr + 2)%dssize;
		wctr++;
	}

	msp_Reset(bc);
	msp_Reset(mt);

	msp_Close(bc);
	msp_Close(mt);

	msp_Cleanup();

	return 0;
}

static msp_FLAGID bcflags[] =
{
	mspF_ENHANCED_MODE,
	mspF_256WORD_BOUNDARY_DISABLE,
	mspF_MESSAGE_GAP_TIMER_ENABLED,
	mspF_INTERNAL_TRIGGER_ENABLED,
	mspF_EXPANDED_BC_CONTROL_WORD,
	0
};

void init_buffer(msp_WORD *buffer, msp_WORD val)
{
	int i;
	for (i = 0; i < 32; i++)
		buffer[i] = val;
}

int configure_bc(msp_DEVHANDLE bc)
{
	msp_FRMHANDLE fr;
	msp_Message M;
	msp_WORD val = 0;
	msp_WORD buf[32];

	init_buffer(buf, val);

	msp_Configure(bc, msp_MODE_BC+msp_MODE_ENHANCED, bcflags, NULL);
	fr = msp_CreateFrame(bc, 1000, 10);
	if (!fr)
	{
		printf("Failed create frame\n");
		return 1;
	}
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 1, 1, 32, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 1, 2, 32, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 1, 3, 32, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 1, 4, 32, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 2, 1, 32, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 3, 1, 32, buf, msp_BCCW_CHANNEL_A)),
			1000);
	msp_AddMessage(fr,
		msp_CreateMessage(bc,
			msp_BCtoRT(&M, 4, 1, 32, buf, msp_BCCW_CHANNEL_A)),
			1000);

	msp_LoadFrame(fr, msp_AUTOREPEAT);
	return 0;
}
