unit rt;

interface

uses access, config;

(***************************************************************
	  
 ***************************************************************)

type
msp_BUFHANDLE = msp_HANDLE;
msp_SUBADDRESS = msp_BYTE;

(***************************************************************
        .   ()
 ***************************************************************)

const
// 
msp_SA_NO_BUFFER              = 0;
msp_SA_SINGLE_BUFFER          = 8;
msp_SA_DOUBLE_BUFFER          = 16;
msp_SA_CIRCULAR_BUFFER_128    = 1;
msp_SA_CIRCULAR_BUFFER_256    = 2;
msp_SA_CIRCULAR_BUFFER_512    = 3;
msp_SA_CIRCULAR_BUFFER_1024   = 4;
msp_SA_CIRCULAR_BUFFER_2048   = 5;
msp_SA_CIRCULAR_BUFFER_4096   = 6;
msp_SA_CIRCULAR_BUFFER_8192   = 7;

//   
//8,16,32 -      
//  .
msp_SA_ROLLOVER_INT                         = 8;
    //      
msp_SA_EOM_INT                              = 16;
    //   
msp_SA_DOUBLE_BUFFERING                     = 32;
    //     
    //:      
    //    2  , 
    // ,    
    //  1
msp_SA_STREAMING                            = 64;
    //   (   RTL)
msp_SA_BUSY									                = 128;
	//      
	//. ,    
	//mspFS_SET_BUSY_VIA_BUFFER_HANDLE

msp_SA_FLAG_mask                            = $F8;

(***************************************************************
      ()  \ 
 ***************************************************************)

const
(*reserved mspSTK_B/A*        $01*)
msp_SA_FREE_RAM             = $02;
msp_SA_ACTIVE_BUFFER        = $04;
msp_SA_INACTIVE_BUFFER      = $08;
msp_SA_SAFE_READ            = $10;
msp_SA_RETURN_IF_BUSY       = $40;
msp_SA_WAIT_IF_BUSY         = $80;
msp_SA_SWAP_BUFFER			    =$100;

(***************************************************************
       
 ***************************************************************)

function msp_CreateBuffer(device: msp_DEVHANDLE;
                          BufferSize: msp_BYTE;
                          BufferFlags: msp_BYTE): msp_BUFHANDLE; cdecl;
//     .
//device  
//BufferSize    -   
//                                  msp_SA_SINGLE_BUFFER
//                                  msp_SA_DOUBLE_BUFFER
//                                  msp_SA_CIRCULAR_BUFFER_128 - msp_SA_CIRCULAR_BUFFER_8192
//BufferFlags -     ,
//                                      ;   
//                                  msp_SA_ROLLOVER_INT
//                                  msp_SA_EOM_INT
//                           msp_SA_DOUBLE_BUFFERING, msp_SA_STREAMING 
//                               

function msp_DestroyBuffer(buffer: msp_BUFHANDLE): msp_ERROR;
// () .

(***************************************************************
	   .
 ***************************************************************)

function msp_SetSABuffer(device: msp_DEVHANDLE;
                         subaddress: msp_SUBADDRESS;
                         dir: msp_DIR;
                         buffer: msp_BUFHANDLE;
                         flags: Integer): msp_ERROR; cdecl;
//     .
//device  
//subaddress  
//dir -  ,   
//  msp_DIR_TX     
//  msp_DIR_RX     
//  msp_DIR_BCST       
//buffer  . (handle)   
//flags    ,   
//  msp_STK_A     lookup table A
//  msp_STK_B     lookup table B;   msp_STK_A;
//              msp_STK_A,  msp_STK_B,  msp_STK_A
//  msp_SA_FREE_RAM     
//  msp_SA_RETURN_IF_BUSY - ,         
//  msp_SA_WAIT_IF_BUSY - ,        
//                                            ( )

function msp_GetSABuffer(device: msp_DEVHANDLE;
                         subaddress: msp_SUBADDRESS;
                         dir: msp_DIR;
                         flags: Integer): msp_BUFHANDLE; cdecl;
//     .
//device  
//subaddress  
//dir -  ,   
//  msp_DIR_TX     
//  msp_DIR_RX     
//  msp_DIR_BCST       
//flags    ,   
//  msp_STK_A    lookup table A
//  msp_STK_B    lookup table B;   msp_STK_A;
//     msp_STK_A,  msp_STK_B,  msp_STK_A
//:  (handle)   .

(***************************************************************
	        "Indirect"
 ***************************************************************)

type
msp_BufConfig = record
  size: msp_BYTE;
  flags: msp_BYTE;
	lowlen: msp_BYTE;
	highlen: msp_BYTE;
end;

msp_SAConfig = record
    rx: msp_BufConfig;
    tx: msp_BufConfig;
    bcst: msp_BufConfig;
end;
Pmsp_SAConfig = ^msp_SAConfig;

msp_SABuffers= record
    rxbuffer: msp_BUFHANDLE;
    txbuffer: msp_BUFHANDLE;
    bcstbuffer: msp_BUFHANDLE;
end;
Pmsp_SABuffers= ^msp_SABuffers;

function msp_SetSAConfig(var config: msp_SAConfig;
    rxsize: msp_BYTE;
    rxflags: msp_BYTE;
    txsize: msp_BYTE;
    txflags: msp_BYTE;
    bcstsize: msp_BYTE;
    bcstflags: msp_BYTE): Pmsp_SAConfig;

function msp_SetSAConfigLeg(var config: msp_SAConfig;
    rxsize: msp_BYTE;
    rxflags: msp_BYTE;
	  rxlowlen: msp_BYTE;
	  rxhighlen: msp_BYTE;
    txsize: msp_BYTE;
    txflags: msp_BYTE;
	  txlowlen: msp_BYTE;
	  txhighlen: msp_BYTE;
    bcstsize: msp_BYTE;
    bcstflags: msp_BYTE;
	  bcstlowlen: msp_BYTE;
	  bcsthighlen: msp_BYTE): Pmsp_SAConfig;

function msp_SetSABuffers(var buffers: msp_SABuffers;
    rxbuffer: msp_BUFHANDLE;
    txbuffer: msp_BUFHANDLE;
    bcstbuffer: msp_BUFHANDLE): Pmsp_SABuffers;

(***************************************************************
	 .
 ***************************************************************)

function msp_ConfigureSA(
                    device: msp_DEVHANDLE;
                    subaddress: msp_SUBADDRESS;
                    rxbuffer: msp_BUFHANDLE;
                    txbuffer: msp_BUFHANDLE;
                    bcstbuffer: msp_BUFHANDLE;
                    flags: Integer): msp_ERROR; cdecl;
//   
//device  
//subaddress  
//txbuffer,
//rxbuffer,
//bcstbuffer   (handle) 
//flags    ,   
//          msp_STK_A     lookup table A
//          msp_STK_B     lookup table B
//          msp_SA_FREE_RAM     

function msp_ConfigureSAIndirect(
                    device: msp_DEVHANDLE;
                    subaddress: msp_SUBADDRESS;
                    config: Pmsp_SAConfig;
                    buffers: Pmsp_SABuffers;
                    flags: Integer): msp_ERROR; cdecl;
//device  
//subaddress  
//config -  
//buffers -  . .. NULL.   NULL,  
//                  ;    config.  - 
//                   msp_NOHANDLE, -   ,  
//                  handle  . .      
//                   (msp_SA_NO_BUFFER), .    NULL.
//flags    ,   
//          msp_STK_A     lookup table A
//          msp_STK_B     lookup table B
//          msp_SA_FREE_RAM     

function msp_ConfigureAllSAIndirect(
                    device: msp_DEVHANDLE;
                    subaddress: msp_SUBADDRESS;
                    config: Pmsp_SAConfig;
                    buffers: Pmsp_SABuffers;
                    flags: Integer): msp_ERROR; cdecl;
//  32 
//device  
//config -   (  32 ),  
//                    msp_ConfigureSAIndirect
//buffers -  32   .    
//                   msp_ConfigureSAIndirect. .. NULL.
//flags    ,   
//          msp_STK_A     lookup table A
//          msp_STK_B     lookup table B
//          msp_SA_FREE_RAM     

(***************************************************************
	\     
 ***************************************************************)

function msp_SetBufferFlag(buffer: msp_BUFHANDLE; flagid: msp_BYTE;
              value: msp_BIT): msp_ERROR; cdecl;
//  .
//    .    (msp_SetSABuffer())
//  .
//buffer - 
//flagid - . ,    msp_SA_XXXX
//value -  ,    0  1

function msp_GetBufferFlag(buffer: msp_BUFHANDLE; flagid: msp_BYTE): msp_BIT; cdecl;
//  .
//   (msp_SetSABuffer()),  
//  
//buffer - 
//flagid - . ,    msp_SA_XXXX
//            msp_SA_DOUBLE_BUFFERING 
//              , 
//           msp_SetBufferFlags (     )

(***************************************************************
	  \ 
 ***************************************************************)

function msp_ReadSAData(sabuffer: msp_BUFHANDLE; buffer: Pmsp_WORD;
  offset: msp_WORD; wordcount: msp_WORD; flags: Integer): msp_WORD; cdecl;
//   
//sabuffer   (handle)    
//offset -  1-    sabuffer
//buffer      
//wordcounut    
//flags        ,
//     :
//  msp_SA_ACTIVE_BUFFER    
//                  (     )
//  msp_SA_INACTIVE_BUFFER    
//                  (     )
//                    msp_SA_DOUBLE_BUFFER   
//                  msp_SA_ACTIVE_BUFFER,  msp_SA_INACTIVE_BUFFER,
//                   -   ,    
//  msp_SA_INACTIVE_BUFFER + msp_SA_SAFE_READ
//                     ,
//                       
//  msp_SA_RETURN_IF_BUSY - ,         
//  msp_SA_WAIT_IF_BUSY - ,        
//                                            ( )
//:    ;   - 0;

function msp_WriteSAData(sabuffer: msp_BUFHANDLE; buffer: Pmsp_WORD;
   offset: msp_WORD; wordcount: msp_WORD; flags: Integer): msp_WORD; cdecl;
//   
//sabuffer   (handle)    
//buffer      
//offset -  1-    sabuffer
//wordcounut    
//flags        ,
//     :
//  msp_SA_ACTIVE_BUFFER    
//                  (     )
//  msp_SA_INACTIVE_BUFFER    
//                  (     )
//                    msp_SA_DOUBLE_BUFFER   
//                  msp_SA_ACTIVE_BUFFER,  msp_SA_INACTIVE_BUFFER,
//                   -   ,    
//  msp_SA_SWAP_BUFFER -      
//					 
//  msp_SA_RETURN_IF_BUSY - ,         
//  msp_SA_WAIT_IF_BUSY - ,        
//:    .

(***************************************************************
	  \  -  
 ***************************************************************)

function msp_ReadSAStream(sabuffer: msp_BUFHANDLE; buffer: Pmsp_WORD;
                                      wordcount: msp_WORD): msp_WORD; cdecl;
//      
//sabuffer   (handle)    
//                  ( msp_SA_CIRCULLAR_BUFFER_XXXXXXX)
//buffer      
//wordcounut    
//:    ;
// = 0       ,
//        

function msp_WriteSAStream(sabuffer: msp_BUFHANDLE; buffer: Pmsp_WORD;
                                       wordcount: msp_WORD): msp_WORD; cdecl;
//      
//sabuffer   (handle)    
//                  ( msp_SA_CIRCULLAR_BUFFER_XXXXXXX)
//buffer      
//wordcounut    
//:    ;
// = 0        ,
//        

function msp_MarkSAStream(sabuffer: msp_BUFHANDLE; flags: Integer): msp_ERROR; cdecl;
//Flags:
const
msp_STREAM_READ     = 1;
msp_STREAM_WRITTEN  = 2;
msp_STREAM_EMPTY    = 3;

function msp_ResetSAStream(sabuffer: msp_BUFHANDLE; offset: msp_WORD): msp_ERROR;
// . . msp_ResetSABuffer

(***************************************************************
	     
 ***************************************************************)

function msp_ResetSABuffer(sabuffer: msp_BUFHANDLE; offset: msp_WORD): msp_ERROR; cdecl;
//       
//sabuffer -  (handle)    
//offset -      
//			 msp_SA_DOUBLE_BUFFER   0  32
//			 msp_SA_CIRCULAR_BUFFER_ - ,     
//			 msp_SA_SINGLE_BUFFER -  

(***************************************************************
	      
 ***************************************************************)

type
msp_ModeCodeInitData = record
    CmdWord: msp_WORD;
    Data: msp_WORD
end;
Pmsp_ModeCodeInitData = ^msp_ModeCodeInitData;

const
msp_LEGAL           = 0;
msp_ILLEGAL         = 1;
msp_BUSY            = 1;
msp_NOT_BUSY        = 0;
msp_ENABLE          = 1;
msp_DISABLE         = 0;

(***************************************************************
	    (illegalizetion)
 ***************************************************************)

function msp_SetIllegalCommands(device: msp_DEVHANDLE;
                rangeCmdWord0, rangeCmdWord1: msp_WORD;
                illegal: msp_BIT): msp_ERROR; cdecl;
///*  
//device - 
//rangeCmdWord0,
//rangeCmdWord1 -   1553,    
//              sampleCmdWord1  
//          ,  sampleCmdWord2 - 
//           :
//          <RT> :  = 31 -     ,
//                             -    ,
//                            
//          <TX> :  /
//          <SA> :   
//          <WC/MC> :  /  (mode code)
//illegal -    

function msp_SetIllegalCommandsDW(device: msp_DEVHANDLE;
                rangeCmdWord0, rangeCmdWord1: msp_WORD;
                illegal: msp_DWORD): msp_ERROR; cdecl;
// msp_SetIllegalCommands(),   <WC/MC> ,
// illegal      32  <WC/MC>

function msp_IsIllegalCommand(device: msp_DEVHANDLE; CmdWord: msp_WORD): msp_BIT; cdecl;
//   

function msp_SetIllegalCommand(device: msp_DEVHANDLE; cmd: msp_WORD;
                illegal: msp_BIT): msp_ERROR;

function msp_SetIllegalCommandsIndirect(
    device: msp_DEVHANDLE;
    ranges: Pmsp_rangeCmdWords;
    count: Integer;
    illegal: msp_BIT): msp_ERROR; cdecl;
///*  
//device - 
//ranges -     (   msp_SetIllegalCommands)
//count -   
//illegal -    

function msp_IllegalizeForBuffer(buffer: msp_BUFHANDLE;
    lowlen, highlen: msp_BYTE): msp_ERROR; cdecl;
///  ,    
//    mspFS_RT_ILLEGALIZE_FOR_BUFFER
//    ,   -
//  
// ,       
//,  - 
//buffer -  
//lowlen - .   
//highlen - .   

(***************************************************************
	   
 ***************************************************************)

function msp_SetBusySA(device: msp_DEVHANDLE;
                rangeCmdWord0, rangeCmdWord1: msp_WORD;
                busy: msp_BIT): msp_ERROR; cdecl;
//    
//device - 
//rangeCmdWord0,
//rangeCmdWord1 -  ,    
//              sampleCmdWord1  
//          ,  sampleCmdWord2 - 
//           :
//          <RT> :  = 31 -     ,
//                             -    ,
//                            
//          <TX> :  /
//          <SA> :   
//busy -    

function msp_GetBusySA(device: msp_DEVHANDLE; CmdWord: msp_WORD): msp_BIT; cdecl;
//   

function msp_SetBusySAIndirect(
    device: msp_DEVHANDLE;
    ranges: Pmsp_rangeCmdWords;
    count: Integer;
    busy: msp_BIT): msp_ERROR; cdecl;
//    
//device - 
//ranges -     (   msp_SetBusySA)
//count -   
//busy -    

(***************************************************************
	    
	 mode code
 ***************************************************************)

function msp_SetModeCodeData(device: msp_DEVHANDLE; CmdWord: msp_WORD;
                             data: msp_WORD): msp_ERROR; cdecl;
//     mode code
//device - 
//CmdWord - ,    . :
//          <RT> :  = 31 -     ,
//                             -    ,
//                            
//          <TX> :  /
//          <MC> :    (mode code)
//data -   

function msp_GetModeCodeData(device: msp_DEVHANDLE; CmdWord: msp_WORD): msp_WORD; cdecl;
//     mode code
//device - 
//CmdWord - ,    .
//                     msp_SetModeCodeData
//:   

function msp_SetModeCodeDataIndirect(
    device: msp_DEVHANDLE;
    data: Pmsp_ModeCodeInitData): msp_ERROR; cdecl;
//      mode code
//device - 
//data -   <, >.   
//                    msp_SetModeCodeData().  
//                   -  CmdWord

function msp_SetModeCodeInt(device: msp_DEVHANDLE; CmdWord: msp_WORD;
                            enable: msp_BIT): msp_ERROR; cdecl;
//     mode code
//device - 
//CmdWord - ,   / 
//                     msp_SetModeCodeData
//enable -    

function msp_GetModeCodeInt(device: msp_DEVHANDLE; CmdWord: msp_WORD): msp_BIT; cdecl;
//      mode code
//device - 
//CmdWord - ,   / 
//                     msp_SetModeCodeData

function msp_SetModeCodeIntIndirect(
    device: msp_DEVHANDLE;
    CmdWords: Pmsp_WORD;
    enable: msp_BIT): msp_ERROR; cdecl;
//      mode code
//device - 
//CmdWords -  ,     msp_SetModeCodeData;
//                             -  0

(***************************************************************
	   
 ***************************************************************)

function msp_CreateStack(device: msp_DEVHANDLE): msp_STKHANDLE; cdecl;
//    (  ).
//     .
//   ,   ,  . ???
//device  
//:  (handle) .

function msp_DestroyStack(stack: msp_STKHANDLE): msp_ERROR;
// .
//stack   (handle) 

function msp_RTLoadStack(stack: msp_STKHANDLE; flags: Integer): msp_ERROR; cdecl;
//      A  B.
//stack   (handle) 
//flags    
//  msp_STK_A     A
//  msp_STK_B     B

function msp_RTLoadStackWithOffset(stack: msp_STKHANDLE; start, flags: Integer): msp_ERROR; cdecl;
//      A  B.
//    
//stack   (handle) 
//start -   
//flags    
//	msp_STK_A     A
//	msp_STK_B     B

//See also msp_ResetStack (config.h)

(***************************************************************
      
 ***************************************************************)

function msp_RTRetrieveMessage(stack: msp_STKHANDLE; index: Integer;
    var buffer: msp_Message): msp_ERROR; cdecl;
// ,  msp_RetrieveMessage,      RT

function msp_RetrieveMCMessage(stack: msp_STKHANDLE; index: Integer;
    var CmdWord, DataWord: msp_WORD): msp_ERROR; cdecl;
//  mode code   .
//stack  
//index        . 
//                  msp_NEXT_MESSAGE,
//                  msp_SAME_MESSAGE
//                  msp_LAST_RECEIVED (. msp_RetrieveMessage())
//CmdWord    (  )
//DataWord    (  )
//   -  "mode code",    msp_ERROR_NOT_MODECODE,
//     .

type
msp_MessageInfo = record
    bsw: msp_WORD;
    timetag: msp_WORD;
    dataAddress: msp_WORD;
    cmdword: msp_WORD;
    mtype: msp_WORD;
    dataWordCount: msp_WORD;
    bufhandle: msp_BUFHANDLE;
    dataOffset: msp_WORD;
end;

function msp_RetrieveMessageInfo(stack: msp_STKHANDLE; index: Integer;
    var info: msp_MessageInfo): msp_ERROR; cdecl;
// msp_RTRetrieveMessage,    
//  " "  - msp_MessageInfo.
// -  ,     SAFE_READ  .

implementation

function msp_CreateBuffer; external 'drtl3.dll' name 'msp_CreateBuffer';

function msp_DestroyBuffer(buffer: msp_BUFHANDLE): msp_ERROR;
begin
  Result := msp_DestroyHandle(buffer)
end;

function msp_SetSABuffer; external 'drtl3.dll' name 'msp_SetSABuffer';
function msp_GetSABuffer; external 'drtl3.dll' name 'msp_GetSABuffer';

function msp_SetSAConfig(var config: msp_SAConfig;
    rxsize: msp_BYTE;
    rxflags: msp_BYTE;
    txsize: msp_BYTE;
    txflags: msp_BYTE;
    bcstsize: msp_BYTE;
    bcstflags: msp_BYTE): Pmsp_SAConfig;
begin
    config.rx.size := rxsize;
    config.rx.flags := rxflags;
    config.rx.lowlen := 1;
    config.rx.highlen := 32;
    config.tx.size := txsize;
    config.tx.flags := txflags;
    config.tx.lowlen := 1;
    config.tx.highlen := 32;
    config.bcst.size := bcstsize;
    config.bcst.flags := bcstflags;
    config.bcst.lowlen := 1;
    config.bcst.highlen := 32;
    Result := @config
end;

function msp_SetSAConfigLeg(var config: msp_SAConfig;
    rxsize: msp_BYTE;
    rxflags: msp_BYTE;
	  rxlowlen: msp_BYTE;
	  rxhighlen: msp_BYTE;
    txsize: msp_BYTE;
    txflags: msp_BYTE;
	  txlowlen: msp_BYTE;
	  txhighlen: msp_BYTE;
    bcstsize: msp_BYTE;
    bcstflags: msp_BYTE;
	  bcstlowlen: msp_BYTE;
	  bcsthighlen: msp_BYTE): Pmsp_SAConfig;
begin
    config.rx.size := rxsize;
    config.rx.flags := rxflags;
    config.rx.lowlen := rxlowlen;
    config.rx.highlen := rxhighlen;
    config.tx.size := txsize;
    config.tx.flags := txflags;
    config.tx.lowlen := txlowlen;
    config.tx.highlen := txhighlen;
    config.bcst.size := bcstsize;
    config.bcst.flags := bcstflags;
    config.bcst.lowlen := bcstlowlen;
    config.bcst.highlen := bcsthighlen;
    Result := @config
end;

function msp_SetSABuffers(var buffers: msp_SABuffers;
    rxbuffer: msp_BUFHANDLE;
    txbuffer: msp_BUFHANDLE;
    bcstbuffer: msp_BUFHANDLE): Pmsp_SABuffers;
begin
    buffers.rxbuffer := rxbuffer;
    buffers.txbuffer := txbuffer;
    buffers.bcstbuffer := bcstbuffer;
    Result := @buffers;
end;

function msp_ConfigureSA; external 'drtl3.dll' name 'msp_ConfigureSA';
function msp_ConfigureSAIndirect; external 'drtl3.dll' name 'msp_ConfigureSAIndirect';
function msp_ConfigureAllSAIndirect; external 'drtl3.dll' name 'msp_ConfigureAllSAIndirect';
function msp_SetBufferFlag; external 'drtl3.dll' name 'msp_SetBufferFlag';
function msp_GetBufferFlag; external 'drtl3.dll' name 'msp_GetBufferFlag';
function msp_ReadSAData; external 'drtl3.dll' name 'msp_ReadSAData';
function msp_WriteSAData; external 'drtl3.dll' name 'msp_WriteSAData';
function msp_ReadSAStream; external 'drtl3.dll' name 'msp_ReadSAStream';
function msp_WriteSAStream; external 'drtl3.dll' name 'msp_WriteSAStream';
function msp_MarkSAStream; external 'drtl3.dll' name 'msp_MarkSAStream';

function msp_ResetSAStream(sabuffer: msp_BUFHANDLE; offset: msp_WORD): msp_ERROR;
begin
  Result := msp_ResetSABuffer(sabuffer, offset)
end;

function msp_ResetSABuffer; external 'drtl3.dll' name 'msp_ResetSABuffer';
function msp_SetIllegalCommands; external 'drtl3.dll' name 'msp_SetIllegalCommands';
function msp_SetIllegalCommandsDW; external 'drtl3.dll' name 'msp_SetIllegalCommandsDW';
function msp_IsIllegalCommand; external 'drtl3.dll' name 'msp_IsIllegalCommand';

function msp_SetIllegalCommand(device: msp_DEVHANDLE; cmd: msp_WORD;
                illegal: msp_BIT): msp_ERROR;
begin
  Result := msp_SetIllegalCommands(device, cmd, cmd, illegal)
end;

function msp_SetIllegalCommandsIndirect; external 'drtl3.dll' name 'msp_SetIllegalCommandsIndirect';
function msp_IllegalizeForBuffer; external 'drtl3.dll' name 'msp_IllegalizeForBuffer';
function msp_SetBusySA; external 'drtl3.dll' name 'msp_SetBusySA';
function msp_GetBusySA; external 'drtl3.dll' name 'msp_GetBusySA';
function msp_SetBusySAIndirect; external 'drtl3.dll' name 'msp_SetBusySAIndirect';
function msp_SetModeCodeData; external 'drtl3.dll' name 'msp_SetModeCodeData';
function msp_GetModeCodeData; external 'drtl3.dll' name 'msp_GetModeCodeData';
function msp_SetModeCodeDataIndirect; external 'drtl3.dll' name 'msp_SetModeCodeDataIndirect';
function msp_SetModeCodeInt; external 'drtl3.dll' name 'msp_SetModeCodeInt';
function msp_GetModeCodeInt; external 'drtl3.dll' name 'msp_GetModeCodeInt';
function msp_SetModeCodeIntIndirect; external 'drtl3.dll' name 'msp_SetModeCodeIntIndirect';
function msp_CreateStack; external 'drtl3.dll' name 'msp_CreateStack';

function msp_DestroyStack(stack: msp_STKHANDLE): msp_ERROR;
begin
  Result := msp_DestroyHandle(stack)
end;

function msp_RTLoadStack; external 'drtl3.dll' name 'msp_RTLoadStack';
function msp_RTLoadStackWithOffset; external 'drtl3.dll' name 'msp_RTLoadStackWithOffset';
function msp_RTRetrieveMessage; external 'drtl3.dll' name 'msp_RTRetrieveMessage';
function msp_RetrieveMCMessage; external 'drtl3.dll' name 'msp_RetrieveMCMessage';
function msp_RetrieveMessageInfo; external 'drtl3.dll' name 'msp_RetrieveMessageInfo';

end.
